% SETUP_LFJC_MODEL
%
% Linear Flexible Joint Cart Lab
%
% SETUP_LFJC_MODEL sets IP02 and LFJC-E system model parameters according to the user-defined configuration.

clear all;

% ############### CONFIGURATION ###############
%Type of IP02 Cart Load: set to 'NO_WEIGHT', 'WEIGHT'
%IP02_WEIGHT_TYPE = 'NO_WEIGHT';
IP02_WEIGHT_TYPE = 'WEIGHT';
% Type of LFJC Load: set to 'NO_WEIGHT', 'ONE_WEIGHT', 'TWO_WEIGHT'
%LFJC_WEIGHT_TYPE = 'NO_WEIGHT';
%LFJC_WEIGHT_TYPE = 'ONE_WEIGHT';
LFJC_WEIGHT_TYPE = 'TWO_WEIGHT';

% Motor Armature Resistance (Ohm)
Rm = 2.6;
% Motor Armature Inductance (H)
Lm = 180e-6;
% Motor Torque Constant (N.m/A)
Kt = 0.00767;
% Motor ElectroMechanical Efficiency [ = Tm * w / ( Vm * Im ) ]
eta_m = 1;
% Motor Back-EMF Constant (V.s/rad)
Km = 0.00767;
% Rotor Inertia (kg.m^2)
Jm = 3.9e-7;
% IP02 Cart Mass, with 3 cable connectors (kg)
Mc2 = 0.57;
% Cart Weight Mass (kg)
Mw = 0.37;
% Planetary Gearbox (a.k.a. Internal) Gear Ratio
Kg = 3.71;
% Planetary Gearbox Efficiency
eta_g = 1;
% Cart Motor Pinion number of teeth
N_mp = 24;
% Motor Pinion Radius (m)
r_mp = 6.35e-3;
% Cart Position Pinion number of teeth
N_pp = 56;
% Position Pinion Radius (m)
r_pp = 14.8e-3;
% Rack Pitch (m/teeth)
Pr = 0.0017;
% Cart Travel (m)
Tc = 0.814;

    if strcmp(IP02_WEIGHT_TYPE, 'NO_LOAD')
        M = Mc2;
        Beq = 4.3;
    elseif strcmp (IP02_WEIGHT_TYPE, 'WEIGHT')
        M = Mc2 + Mw;
        Beq = 5.4;
    end
    
% Lumped Mass of the Cart System (accounting for the rotor inertia)
Jeq = M + eta_g * Kg^2 * Jm / r_mp^2;

% Position Pinion number of teeth
N_pp2 = 56;
% Position Pinion Radius (m)
r_pp2 = 14.8e-3;
% Position Pinion (or Rack) Pitch (m/tooth)
P_pp2 = 0.0017;
% Equivalent Spring Stiffness (N/m)
Ks = 142;
% Spring Assembly Mass (i.e. spring + 2 fixtures + 8 screws) (kg) 
Ms = 0.145;
% Adjusted Total Mass of the IP02 Cart System (kg)
Jeq = Jeq + Ms / 2;
% LFJC-E Mass (cart alone) (kg)
m_jc = 0.220;
% Cart Encoder Resolution (m/count)
K_EC_LFJC = P_pp2 * N_pp2 / ( 4 * 1024 ); % = 22.7485 um/count
% no pendulum encoder
K_EP_LFJC = 0;
% Cart Mass Adjustment (kg)
m_jc = m_jc + Ms / 2;
% LFJC Weight Mass (kg)
Mw_jc = 0.125;
% Cart Total Mass (kg)
% estimate Beq2
if strcmp ( LFJC_WEIGHT_TYPE, 'NO_WEIGHT' )
    Beq_jc = 0;
elseif strcmp ( LFJC_WEIGHT_TYPE, 'ONE_WEIGHT' )
    % Cart Total Mass (kg)
    m_jc = m_jc + Mw_jc;
    Beq_jc = 0;
elseif strcmp ( LFJC_WEIGHT_TYPE, 'TWO_WEIGHT' )
    % Cart Total Mass (kg)
    m_jc = m_jc + 2 * Mw_jc;
    Beq_jc = 2.2;
else
    error( 'Error: Set the LFJC load configuration.' )
end

% ##### USER-DEFINED MODEL PARAMETERS #####
% Specify perticular equivalent spring stiffness, Ks, and equivalent
% damping, Beq_jc, parameters:

%Ks = 0;
%Beq_jc = 0;

%State-Space Model
A = [0 0 1 0;
     0 0 0 1;
     -Ks/Jeq Ks/Jeq -Beq/Jeq 0;
     Ks/m_jc -Ks/m_jc 0 -Beq_jc/m_jc];

B = [0; 0; 1/Jeq; 0];
C = eye(2,4);
D = zeros(2,1);

%Actuator Dynamics
A(3,3) = A(3,3) - B(3)*eta_g*Kg^2*eta_m*Kt*Km/r_mp^2/Rm;
A(4,3) = A(4,3) - B(4)*eta_g*Kg^2*eta_m*Kt*Km/r_mp^2/Rm
B = eta_g*Kg*eta_m*Kt/r_mp/Rm*B